//
//  MLYDevice.m
//  minilytic iOS SDK
//
//  Created by Jean-Christophe Amiel on 5/27/13.
//  Copyright (c) 2013 Manbolo. All rights reserved.
//

#import "MLYDevice.h"
#include <sys/sysctl.h>

#if !__has_feature(objc_arc)
#pragma message("minilytic iOS SDK use ARC. If your project is not ARC (it's time to change), just use -fobjc-arc compile flags on MLY*.m files.")
#endif


@implementation MLYDevice

+ (MLYDevice *)defaultDevice {
    static dispatch_once_t onceToken;
    static MLYDevice *device = nil;
    dispatch_once(&onceToken, ^{
        device = [[MLYDevice alloc] init];
    });
    return device;
}


- (NSString *)sysInfoByName:(char *)typeSpecifier
{
    size_t size;
    sysctlbyname(typeSpecifier, NULL, &size, NULL, 0);
    
    char *answer = malloc(size);
    sysctlbyname(typeSpecifier, answer, &size, NULL, 0);
    
    NSString *results = [NSString stringWithCString:answer encoding: NSUTF8StringEncoding];
    
    free(answer);
    return results;
}

- (NSString *)platform
{
    return [self sysInfoByName:"hw.machine"];
}


- (NSString *)hardwareModel
{
    return [self sysInfoByName:"hw.model"];
}

- (NSString *)systemVersion
{
    return [UIDevice currentDevice].systemVersion;
}

- (NSString *)systemName
{
    return [UIDevice currentDevice].systemName;
}

- (NSString *)language
{
    NSArray *preferredLanguages = [NSLocale preferredLanguages];
    if ([preferredLanguages count]){
        return preferredLanguages[0];
    }
    else {
        return nil;
    }
}



@end
