//
//  MLYHTTPRequest.m
//  minilytic iOS SDK
//
//  Copyright (c) 2013 Manbolo. All rights reserved.
//

#import "MLYHTTPRequest.h"

#if !__has_feature(objc_arc)
#pragma message("minilytic iOS SDK use ARC. If your project is not ARC (it's time to change), just use -fobjc-arc compile flags on MLY*.m files.")
#endif


@interface MLYHTTPRequest ( )

@property(nonatomic,assign) UIBackgroundTaskIdentifier backgroundTaskId;
@property(nonatomic,strong) NSMutableURLRequest *request;
@property(nonatomic,strong) NSURLConnection *connection;
@property(nonatomic,assign) MLYHTTPRequestState state;

@end


@implementation MLYHTTPRequest{
    MLYHTTPRequestState _state;
}

#pragma mark - Entry point

- (id)init
{
    self = [super init];
    if (self){
        _usingLocalhost = NO;
        _request = [self requestWithURLString:@"https://minilytic.com/api/v1/hits/"];
        _state = MLYHTTPRequestStateReady;
    }
    return self;
}

- (void)dealloc
{
    [_connection cancel];
    _connection = nil;
}

- (NSMutableURLRequest *)requestWithURLString:(NSString*)urlString
{
    NSURL *url = [NSURL URLWithString:urlString];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    [request setHTTPMethod:@"POST"];
    [request setValue:@"gzip" forHTTPHeaderField:@"Content-Encoding"];
    [request setValue:@"text/json" forHTTPHeaderField:@"Content-Type"];
    [request setCachePolicy:NSURLRequestReloadIgnoringLocalCacheData];
    return request;
}


- (MLYHTTPRequestState)state
{
    return _state;
}

- (void)setState:(MLYHTTPRequestState)newState
{
    switch (newState) {
        case MLYHTTPRequestStateReady:
            [self willChangeValueForKey:@"isReady"];
            break;
        case MLYHTTPRequestStateExecuting:
            [self willChangeValueForKey:@"isReady"];
            [self willChangeValueForKey:@"isExecuting"];
            break;
        case MLYHTTPRequestStateFinished:
            [self willChangeValueForKey:@"isExecuting"];
            [self willChangeValueForKey:@"isFinished"];
            break;
    }
    
    _state = newState;
    
    switch (newState) {
        case MLYHTTPRequestStateReady:
            [self didChangeValueForKey:@"isReady"];
            break;
        case MLYHTTPRequestStateExecuting:
            [self didChangeValueForKey:@"isReady"];
            [self didChangeValueForKey:@"isExecuting"];
            break;
        case MLYHTTPRequestStateFinished:
            [self didChangeValueForKey:@"isExecuting"];
            [self didChangeValueForKey:@"isFinished"];
            break;
    }
}


- (void)setUsingLocalhost:(BOOL)usingLocalhost
{
    _usingLocalhost = usingLocalhost;
    _request = self.isUsingLocalhost ?
        [self requestWithURLString:@"http://localhost:8000/api/v1/hits/"] :
        [self requestWithURLString:@"https://minilytic.com/api/v1/hits/"];
}

- (void)main
{
    @autoreleasepool {
        [self start];
    }
}

- (void)start
{
    _backgroundTaskId = [[UIApplication sharedApplication] beginBackgroundTaskWithExpirationHandler:^{
        dispatch_async(dispatch_get_main_queue(), ^{
            if (_backgroundTaskId != UIBackgroundTaskInvalid){
                [[UIApplication sharedApplication] endBackgroundTask:_backgroundTaskId];
                _backgroundTaskId = UIBackgroundTaskInvalid;
                [self cancel];
            }
        });
    }];
    
    if(!self.isCancelled) {
        [_request setHTTPBody:_body];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            _connection = [[NSURLConnection alloc] initWithRequest:_request
                                                          delegate:self
                                                  startImmediately:NO];
            
            [_connection scheduleInRunLoop:[NSRunLoop currentRunLoop]
                                   forMode:NSRunLoopCommonModes];
            
            [_connection start];
            
        });
        self.state = MLYHTTPRequestStateExecuting;
    }
    else {
        self.state = MLYHTTPRequestStateFinished;
        [self endBackgroundTask];
    }
}


- (void)endBackgroundTask
{
    dispatch_async(dispatch_get_main_queue(), ^{
        if (_backgroundTaskId != UIBackgroundTaskInvalid) {
            [[UIApplication sharedApplication] endBackgroundTask:_backgroundTaskId];
            _backgroundTaskId = UIBackgroundTaskInvalid;
        }
    });
}

#pragma mark - NSOperation

- (BOOL)isConcurrent
{
    return YES;
}

- (BOOL)isReady
{
    return ((self.state == MLYHTTPRequestStateReady) && [super isReady]);
}

- (BOOL)isFinished
{
    return (self.state == MLYHTTPRequestStateFinished);
}

- (BOOL)isExecuting
{
    
    return (self.state == MLYHTTPRequestStateExecuting);
}



- (void)cancel
{
    if ([self isFinished])
        return;
    
    @synchronized(self) {
        [_connection cancel];
        
        if(self.state == MLYHTTPRequestStateExecuting){
            self.state = MLYHTTPRequestStateFinished;
        }
        [self endBackgroundTask];
    }
    [super cancel];
}

#pragma mark - NSURLConnection delegates

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    self.state = MLYHTTPRequestStateFinished;
    _body = nil;
    [self endBackgroundTask];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    self.state = MLYHTTPRequestStateFinished;
    _body = nil;
    [self endBackgroundTask];
}





@end
